@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # coding: utf-8
 
-# In[82]:
+# In[2]:
 
 
 try:
@@ -12,10 +12,10 @@ except:
         pass
 
 
-# In[83]:
+# In[3]:
 
 
-version = '0.47'
+version = '0.49'
 
 import re
 import tempfile
@@ -32,7 +32,7 @@ import stat
 
 # # Helper Functions
 
-# In[84]:
+# In[4]:
 
 
 def is_testing():
@@ -42,7 +42,7 @@ def is_testing():
         return False
 
 
-# In[85]:
+# In[5]:
 
 
 def isnotebook():
@@ -53,14 +53,7 @@ def isnotebook():
     return False
 
 
-# def is_testing():
-#     try:
-#         return file_name == 'lx.ipynb'
-#     except:
-#         pass
-#     return False
-
-# In[86]:
+# In[6]:
 
 
 def testPrint(*args):
@@ -70,7 +63,7 @@ def testPrint(*args):
 
 # # Requiremtns
 
-# In[87]:
+# In[7]:
 
 
 requirements = []
@@ -99,14 +92,14 @@ from IPython.core.display import display, HTML
 
 # # Lists
 
-# In[88]:
+# In[8]:
 
 
 def listsGetPairsFromTwoLists(list1, list2):
     return list(zip(list1, list2))
 
 
-# In[89]:
+# In[9]:
 
 
 if is_testing():
@@ -116,14 +109,14 @@ if is_testing():
     print(listsGetPairsFromTwoLists(listAbcd, list1234))
 
 
-# In[90]:
+# In[10]:
 
 
 def listSequentialOverlappingPairs(l):
     return list(zip(l[:-1], l[1:]))
 
 
-# In[91]:
+# In[11]:
 
 
 if is_testing():
@@ -131,9 +124,15 @@ if is_testing():
     print(listSequentialOverlappingPairs(list1234))
 
 
+# In[ ]:
+
+
+
+
+
 # # List of Strings
 
-# In[92]:
+# In[12]:
 
 
 regex_cache = {}
@@ -143,35 +142,35 @@ def sListFilter(l, re_include=".*", re_exclude='^\b$'):
     return [e for e in l if regex_cache[re_include].search(e) and not regex_cache[re_exclude].search(e)]
 
 
-# In[93]:
+# In[13]:
 
 
 if is_testing():
     sListFilter(['good x', 'other good x', 'without the letter before y'])
 
 
-# In[94]:
+# In[14]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[95]:
+# In[15]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[96]:
+# In[16]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[97]:
+# In[17]:
 
 
 def lCsv(l, force_oneline=False):
@@ -187,21 +186,21 @@ def lCsv(l, force_oneline=False):
     return ", ".join([str(e) for e in l])
 
 
-# In[98]:
+# In[18]:
 
 
 if is_testing():
     print(lCsv(['a', 'b', 1, 3.14]))
 
 
-# In[99]:
+# In[19]:
 
 
 if is_testing():
     print(lCsv([['a', 'b'], 1, 3.14]))
 
 
-# In[100]:
+# In[20]:
 
 
 if is_testing():
@@ -210,7 +209,7 @@ if is_testing():
 
 # # Text files
 
-# In[101]:
+# In[21]:
 
 
 def txtread(path):
@@ -226,7 +225,7 @@ def txtwrite(path, txt):
         return f.write(txt)
 
 
-# In[102]:
+# In[22]:
 
 
 if is_testing():
@@ -235,7 +234,7 @@ if is_testing():
 
 # # List of Strings
 
-# In[103]:
+# In[23]:
 
 
 regex_cache = {}
@@ -247,44 +246,128 @@ def sListFilter(listOfStrings, re_include=".*", re_exclude='^\b$'):
 
 # ### Examples
 
-# In[104]:
+# In[24]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y']))
 
 
-# In[105]:
+# In[25]:
 
 
 if is_testing():
     print(sListFilter(['good x', 'other good x', 'without the letter before y'], re_include='x'))
 
 
-# In[106]:
+# In[26]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+'))
 
 
-# In[107]:
+# In[27]:
 
 
 if is_testing():
     print(sListFilter(['not enough x', 'enough xx', 'enough xx and also z'], re_include='xx+', re_exclude='z'))
 
 
-# In[108]:
+# In[28]:
 
 
 if is_testing():
     print(txtread('https://arxiv.org'))
 
 
+# # GCP Google Cloud Platform
+
+# In[29]:
+
+
+import warnings
+warnings.filterwarnings("ignore", "Your application has authenticated using end user credentials from Google Cloud SDK")
+
+def _gsPathSplit(path):
+    bucket = path.split("//")[1].split("/")[0]
+    prefix = "/".join(path.split("//")[1].split("/")[1:])
+    return bucket, prefix
+
+def _gsClient():
+    from google.cloud import storage
+    client = storage.Client()
+    
+def _gsBlob(path):
+    from google.cloud import storage
+    bucket_name, prefix = _gsPathSplit(path)
+    storage_client = storage.Client()
+    bucket = storage_client.bucket(bucket_name)
+    blob = bucket.blob(prefix)
+    return blob
+
+
+# In[30]:
+
+
+def gsGsToHttp(path):
+    return path.replace("gs://", "https://storage.cloud.google.com/")
+
+def gsHttpToGs(path):
+    return path.replace("https://storage.cloud.google.com/", "gs://").split("?")[0] # "?" is for possible "?authuser=0"
+
+
+# In[31]:
+
+
+def gsLs(path):
+    from google.cloud import storage
+    bucket, prefix = _gsPathSplit(path)
+    client = storage.Client().bucket(bucket)
+    return ['gs://' + bucket + '/' + blob.name for blob in client.list_blobs(prefix=prefix)]
+
+
+# In[32]:
+
+
+def gsUploadFile(gsPath, path):
+    blob = _gsBlob(gsPath)
+    blob.upload_from_filename(path)
+
+
+# In[33]:
+
+
+def gsTxtread(path):
+    from google.cloud import storage
+    bucket_name, source_blob_name = _gsPathSplit(path)
+    storage_client = storage.Client()
+    bucket = storage_client.bucket(bucket_name)
+    blob = bucket.blob(source_blob_name)
+    return blob.download_as_string().decode()
+
+
+# In[34]:
+
+
+def gsImwrite(path, img):
+    blob = _gsBlob(path)
+    _, encimg = cv2.imencode('.png', img)
+    blob.upload_from_string(encimg.tobytes())
+
+
+# In[35]:
+
+
+def gsImread(path):
+    blob = _gsBlob(path)
+    raw = np.asarray(bytearray(blob.download_as_string()), dtype="uint8")
+    return cv2.imdecode(raw, 1)
+
+
 # # Images
 
-# In[109]:
+# In[36]:
 
 
 def imread(path):
@@ -293,6 +376,8 @@ def imread(path):
     elif 'http://' in path or 'https://' in path:
         raw = np.asarray(bytearray(urlopen(path).read()), dtype="uint8")
         img = cv2.imdecode(raw, cv2.IMREAD_COLOR)
+    elif 'gs://' in path:
+        return gsImread(path)
     elif os.path.isfile(path):
         img = cv2.imread(path, cv2.IMREAD_COLOR)
     return img[:, :, [2, 1, 0]] # This converts the cv2 colors to RGB
@@ -304,6 +389,8 @@ def imsread(paths):
     return [imread(path) for path in paths]
 
 def imwrite(path, img):
+    if "gs://" in path:
+        gsImwrite(path, img)
     os.makedirs(os.path.dirname(path), exist_ok=True)
     cv2.imwrite(path, img[:, :, [2, 1, 0]])
     
@@ -314,7 +401,7 @@ def impad(img, top=0, bottom=0, left=0, right=0, color=255):
     return np.pad(img, [(top, bottom), (left, right), (0, 0)], 'constant', constant_values=color)
 
 
-# In[110]:
+# In[37]:
 
 
 def imshow(array, scale=1):
@@ -329,7 +416,7 @@ def imshow(array, scale=1):
         cv2.waitKey()
 
 
-# In[111]:
+# In[38]:
 
 
 if is_testing():
@@ -337,14 +424,14 @@ if is_testing():
     imshow(img)
 
 
-# In[112]:
+# In[39]:
 
 
 if is_testing():
     imshow('http://via.placeholder.com/70.png')
 
 
-# In[113]:
+# In[40]:
 
 
 def imscaleNN(img, s):
@@ -354,7 +441,7 @@ def imscaleBic(img, s):
     return cv2.resize(img, None, fx=s, fy=s, interpolation=cv2.INTER_CUBIC)
 
 
-# In[114]:
+# In[41]:
 
 
 if is_testing():
@@ -373,33 +460,55 @@ if is_testing():
         imshow(out_large_interbic)
 
 
-# In[115]:
+# In[42]:
+
+
+def imRepeat(img, height, width):
+    height_old, width_old, _ = img.shape
+
+    _ceil = lambda x: int(np.ceil(x))
+
+    img = np.concatenate([img]*_ceil(height/height_old), axis=0)[:height]
+    img = np.concatenate([img]*_ceil(width/width_old), axis=1)[:, :width]
+    
+    return img
+
+
+# In[43]:
+
+
+if is_testing():
+    img_random_color_3x5 = (np.random.rand(3, 5, 3)*255).astype(np.uint8)
+    imshow(imRepeat(img_random_color_3x5, 7, 11), scale=10)
+
+
+# In[44]:
 
 
 def imNewWhite(height, width):
     return np.ones((height, width, 3)).astype(np.uint8) * 255
 
 
-# In[116]:
+# In[45]:
 
 
 if is_testing(): imshow(imNewWhite(10, 500) - 30)
 
 
-# In[117]:
+# In[46]:
 
 
 def imNewBlack(height, width):
     return np.zeros((height, width, 3)).astype(np.uint8)
 
 
-# In[118]:
+# In[47]:
 
 
 if is_testing(): imshow(imNewBlack(10, 500))
 
 
-# In[119]:
+# In[48]:
 
 
 def imAddNoiseGauss(img, std):
@@ -411,7 +520,7 @@ def imAddNoiseGauss(img, std):
 if is_testing(): imshow(imAddNoiseGauss(imread('http://via.placeholder.com/150.png'), 10))
 
 
-# In[120]:
+# In[49]:
 
 
 def imJpgDegradation(img, quality):
@@ -424,7 +533,7 @@ def imJpgDegradation(img, quality):
 if is_testing(): imshow(imJpgDegradation(imread('http://via.placeholder.com/150.png'), 15))
 
 
-# In[121]:
+# In[50]:
 
 
 def imCropCenter(img, size):
@@ -439,35 +548,35 @@ def imCropCenter(img, size):
     return img[h_start:h_end, w_start:w_end]
 
 
-# In[122]:
+# In[51]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 10))
 
 
-# In[123]:
+# In[52]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 49))
 
 
-# In[124]:
+# In[53]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 50))
 
 
-# In[125]:
+# In[54]:
 
 
 if is_testing(): 
     imshow(imCropCenter(imread('http://via.placeholder.com/40.png'), 100))
 
 
-# In[126]:
+# In[55]:
 
 
 def imGallery(imgs, pad=0):
@@ -498,7 +607,7 @@ def imGallery(imgs, pad=0):
     return out
 
 
-# In[127]:
+# In[56]:
 
 
 if is_testing():
@@ -506,49 +615,49 @@ if is_testing():
     listOfDummyImagesBlackToWhite = [imNewBlack(10, 10) + 25 * i for i in range(10)]
 
 
-# In[128]:
+# In[57]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:1]))
 
 
-# In[129]:
+# In[58]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2]))
 
 
-# In[130]:
+# In[59]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:2], pad=1))
 
 
-# In[131]:
+# In[60]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:5], pad=1))
 
 
-# In[132]:
+# In[61]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:9], pad=1))
 
 
-# In[133]:
+# In[62]:
 
 
 if is_testing():
     imshow(imGallery(listOfDummyImagesBlackToWhite[:10], pad=1))
 
 
-# In[134]:
+# In[63]:
 
 
 if is_testing():
@@ -558,14 +667,14 @@ if is_testing():
 
 # # Files
 
-# In[135]:
+# In[64]:
 
 
 def fiFindByWildcard(wildcard):
     return natsort.natsorted(glob.glob(wildcard, recursive=True))
 
 
-# In[136]:
+# In[65]:
 
 
 if is_testing():
@@ -594,7 +703,7 @@ if is_testing():
         imshow(imGallery(imgs))
 
 
-# In[137]:
+# In[66]:
 
 
 def fiRemove(path):
@@ -622,7 +731,7 @@ def fiAgeHours(path):
     return (time.time() - os.stat(path)[stat.ST_MTIME]) / 3600
 
 
-# In[138]:
+# In[67]:
 
 
 if is_testing():
@@ -656,7 +765,7 @@ if is_testing():
 
 # # Jupyter
 
-# In[139]:
+# In[68]:
 
 
 def jnNbFullWidth():
@@ -670,7 +779,7 @@ def jnPandasSettings():
     pd.set_option('display.max_colwidth', None)
 
 
-# In[140]:
+# In[69]:
 
 
 def jnImageRenderingPixelated():
@@ -682,7 +791,7 @@ def jnImageRenderingAuto():
     display(HTML('<style type = text/css> img { image-rendering: auto; } </style>'))
 
 
-# In[141]:
+# In[70]:
 
 
 def jnDefault():
@@ -694,7 +803,7 @@ if is_testing():
     jnDefault()
 
 
-# In[142]:
+# In[71]:
 
 
 def jnHtmlRow(strList, width=100):
@@ -707,7 +816,7 @@ if is_testing():
     
 
 
-# In[143]:
+# In[72]:
 
 
 def jnImgToHtmlB64(img):
@@ -731,7 +840,7 @@ def jnImageRow(imgs, names=None, width=100):
     display(HTML(html))
 
 
-# In[144]:
+# In[73]:
 
 
 if is_testing():
@@ -739,7 +848,7 @@ if is_testing():
    
 
 
-# In[145]:
+# In[74]:
 
 
 
@@ -749,7 +858,7 @@ if is_testing():
 
 # # Shell
 
-# In[146]:
+# In[75]:
 
 
 def shell(cmd):
@@ -764,7 +873,7 @@ def shell(cmd):
 
 # # GIT
 
-# In[147]:
+# In[76]:
 
 
 def gitGetHash():
@@ -774,7 +883,7 @@ if is_testing():
     print(gitGetHash())
 
 
-# In[148]:
+# In[77]:
 
 
 def gitGetBranch():
@@ -784,7 +893,7 @@ if is_testing():
     print(gitGetBranch())
 
 
-# In[149]:
+# In[78]:
 
 
 def gitGetMessage():
@@ -794,7 +903,7 @@ if is_testing():
     print(gitGetMessage())
 
 
-# In[150]:
+# In[79]:
 
 
 def gitGetAllFiles():
@@ -804,7 +913,7 @@ if is_testing():
     print(gitGetAllFiles())
 
 
-# In[151]:
+# In[80]:
 
 
 def gitListByTime(path=None):
@@ -820,7 +929,7 @@ if is_testing():
     print(gitListByTime()[-3:])
 
 
-# In[152]:
+# In[81]:
 
 
 def shellSortDirsBySize():
@@ -832,20 +941,14 @@ if is_testing():
 
 # # Package Files
 
-# In[ ]:
-
-
-
-
-
-# In[153]:
+# In[82]:
 
 
 if is_testing():
     package_files = {}
 
 
-# In[154]:
+# In[83]:
 
 
 def shell(cmd):
@@ -853,7 +956,7 @@ def shell(cmd):
     return result.stdout.decode()
 
 
-# In[155]:
+# In[84]:
 
 
 if is_testing():
@@ -885,7 +988,7 @@ setup(
 )""".format(requirements=", ".join(["'" + r + "'" for r in requirements]), version=version)
 
 
-# In[156]:
+# In[85]:
 
 
 if is_testing():
@@ -895,7 +998,7 @@ description-file = README.md
 """
 
 
-# In[157]:
+# In[86]:
 
 
 if is_testing():
@@ -918,14 +1021,14 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 SOFTWARE."""
 
 
-# In[158]:
+# In[87]:
 
 
 if is_testing():
     package_files['README.md'] = ""
 
 
-# In[159]:
+# In[88]:
 
 
 if is_testing():
@@ -934,7 +1037,7 @@ if is_testing():
     
 
 
-# In[160]:
+# In[89]:
 
 
 if is_testing():
@@ -945,7 +1048,7 @@ lx/__init__.py
 """
 
 
-# In[161]:
+# In[90]:
 
 
 if is_testing():
